/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */


package org.codehaus.groovy.transform.traitx

import groovy.test.GroovyTestCase

class Groovy7269Bug extends GroovyTestCase {
    void testTraitWithMetaClassMod() {
        assertScript '''
            trait SomeTrait {
                String title
            }

            class Widget implements SomeTrait {}

            Widget.metaClass.getTitle = {
                'Some Default Title'
            }
            def w = new Widget()
            w.title = 'Some New Title'
            assert w.title == 'Some Default Title'
        '''
    }

    // GROOVY-7308
    void testMixOfStaticAndNotStaticPropertiesOfSameName() {
        assertScript '''class Widget {

    static Integer someNumber = 42
}

Widget.metaClass.getSomeNumber = {
    2112
}

def widget = new Widget()

assert widget.someNumber == 2112


        '''
    }
}

